"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModuleFederationPlugin = void 0;
class ModuleFederationPlugin {
    constructor(_options) {
        this._options = _options;
    }
    apply(compiler) {
        const { webpack } = compiler;
        new webpack.EntryPlugin(compiler.context, defaultImplementation(this._options, compiler), { name: undefined }).apply(compiler);
        new webpack.container.ModuleFederationPluginV1({
            ...this._options,
            enhanced: true
        }).apply(compiler);
    }
}
exports.ModuleFederationPlugin = ModuleFederationPlugin;
function defaultImplementation(options, compiler) {
    var _a, _b, _c;
    function extractUrlAndGlobal(urlAndGlobal) {
        const index = urlAndGlobal.indexOf("@");
        if (index <= 0 || index === urlAndGlobal.length - 1) {
            return null;
        }
        return [
            urlAndGlobal.substring(index + 1),
            urlAndGlobal.substring(0, index)
        ];
    }
    function getExternalTypeFromExternal(external) {
        if (/^[a-z0-9-]+ /.test(external)) {
            const idx = external.indexOf(" ");
            return [external.slice(0, idx), external.slice(idx + 1)];
        }
        return null;
    }
    function getExternal(external, defaultExternalType) {
        const result = getExternalTypeFromExternal(external);
        if (result === null) {
            return [defaultExternalType, external];
        }
        return result;
    }
    const runtimeTemplate = compiler.webpack.Template.getFunctionContent(require("./default.runtime.js"));
    const remotes = [];
    const remoteType = options.remoteType ||
        (options.library ? options.library.type : "script");
    for (let [key, remote] of Object.entries((_a = options.remotes) !== null && _a !== void 0 ? _a : {})) {
        const [externalType, external] = getExternal(typeof remote === "string" ? remote : remote.external, remoteType);
        const shareScope = (typeof remote !== "string" && remote.shareScope) ||
            options.shareScope ||
            "default";
        if (externalType === "script") {
            const [url, global] = extractUrlAndGlobal(external);
            remotes.push({
                alias: key,
                name: global,
                entry: url,
                externalType,
                shareScope
            });
        }
        else {
            remotes.push({
                alias: key,
                name: undefined,
                entry: undefined,
                externalType,
                shareScope
            });
        }
    }
    const runtimePlugins = (_b = options.runtimePlugins) !== null && _b !== void 0 ? _b : [];
    const pluginImports = runtimePlugins.map(p => `require(${JSON.stringify(p)})`);
    const implementationPath = (_c = options.implementation) !== null && _c !== void 0 ? _c : require.resolve("@module-federation/webpack-bundler-runtime");
    let implementation = runtimeTemplate
        .replace("$RUNTIME_PACKAGE_PATH$", JSON.stringify(implementationPath))
        .replace("$ALL_REMOTES$", JSON.stringify(remotes))
        .replace("$INITOPTIONS_PLUGINS$", `[${pluginImports.join(", ")}]`);
    return `data:text/javascript,${implementation}`;
}
//# sourceMappingURL=ModuleFederationPlugin.js.map